/*testprogram for reconstruction */
#include <fstream.h>
#include <stdio.h>
#include <time.h>

#include "../datalib/vector.h"
#include "../datalib/volume.h"
#include "../mathlib/statistic.h"
#include "../mathlib/transform.h"
#include "../mathlib/moments.h"


#include "../errorcode.h"

#include "../datalib/datacube.h"


#include "../reconstructionlib/multicompartmentlist.h"

#include "command_line_parser.h"

using namespace std;
using namespace pramodx;


int callBack(int percentage)
{
  cout << percentage << "% done" << endl;
  return 0;
}



int
main(int argc, char **argv)
{
   double abslow=0.08;
 double abshigh=0.4;
 double eps=0.05;


  DataCube cube(3);
    FilterOptions *filterOptions = new FilterOptions;
    command_line_parser parser(argc, argv);
    
    if (argc<2) 
    {
      cout << "Usage: segment [Options] inputfilename.tiff  outputfilename.tiff , -h for more options" << endl;
      return 0;
    };
    


for (int i=0;i<parser.get_n_options();i++)
    {
        option o=parser.get_option(i);
        if (strcmp(o.get_option(),"h")==0 || parser.get_n_filenames()==0)
        {
            cout << "Usage: segmt [Options] filename.tif  output.tif" << endl;
            cout << "Options(optional)" << endl;
            cout << "-------" << endl;
	    // cout << "-lo:CFD , load cfd in stead of nih" << endl;
            cout << "-eps:value   eps higher than neigbours goodguess:0.05" << endl;
            cout << "-absl:value  Set absolute low threshold to value(0..1) good guess:0.08" << endl;
            cout << "-absh:value  Set absolute high threshold to value(0..1)good guess:0.4" << endl;
            cout << endl;
            return 0;
        };
       
        if (strcmp(o.get_option(),"eps")==0)
        {
            float help;
            if (sscanf (o.get_value(),"%f",&(help))==EOF) continue;
            eps=help;
            cout << "eps=" << eps << endl;
        };
        if (strcmp(o.get_option(),"absl")==0)
        {
            float help;
            if (sscanf (o.get_value(),"%f",&(help))==EOF) continue;
            abslow=help;
            cout << "absoluteLowThreshold=" << abslow << endl;
        };
        if (strcmp(o.get_option(),"absh")==0)
        {
            float help;
            if (sscanf (o.get_value(),"%f",&(help))==EOF) continue;
            abshigh=help;
            cout << "absoluteLowHighhold=" << abshigh << endl;
        };

    };
    


   
  char inputfilename[1024];
  sprintf(inputfilename,parser.get_filename(0));
  cout << "Input File: " << inputfilename << endl;
  char outputfilename[1024];
  sprintf(outputfilename,parser.get_filename(1));
  cout << "Out put file:" << outputfilename << endl;
  
 int result=cube.ReadFromTIFF(inputfilename);
 if (result!=0 && result!=NO_SPACEINGS_TAGS)
 {
 	cout << "Error loading file " << inputfilename << endl;
 	return 1;
 }

 Transform trans(&cube);

 DataCube *target=new DataCube(cube);

 trans.StructuralThresholding(abslow, abshigh, eps, &target);
  
    
    
  cout << "Finsed Filtering, write output file" << endl << flush; 
  if (target->WriteToTIFF(outputfilename))
  {
  	cout << "Error writing file" << outputfilename << endl;
  	return 1;
  };
  
  
  
  return 0;
}


