/*
 *  vector1.h
 *  DendriteTracker
 *
 *  Created by Philip Broser on 24.02.06.
 *  Copyright 2006 mpimf All rights reserved.
 *
 */


#ifndef VECTOR1_HPP
#define VECTOR1_HPP


//#include <math.h>
//#include <stream.h>
#include <cmath>
#include <iostream>
#include <sstream>


namespace bla //basic linear algebra
{

template <class T>
class Vector
{
	
public:
	//Initalizer
	Vector(){length=0; maxlength=0; elements = NULL;}
	Vector(const int l);
	Vector(const Vector<T>& v1);
	template <class S> Vector(const Vector<S>& v1);
	~Vector();
	
	//Modify
	int Reset(const Vector<T> & vec);
	int Reset(const int newL);
	int ExtendTo(const int );
	
	
	//GetMethods
	int GetLength () const {return(length);}
	int Length () const {return(length);}	
	int GetElement (const int, T&) const;
	T& GetElement(const int) const;
	T& Get(const int) const;
	
	//SetMethods
	int SetZero();
	int SetOne();
	int SetElement(const int, const T&);
	int Set(const T&);
	
	
	//ArithmeticOpps  
	int Add(const int, const T&);
	int Add(const T&);
	int Multiply(const int, const T&);
	int Multiply(const T&);
	//Operators
	
	Vector<T> operator+ (const Vector<T> &vec1);
	Vector<T> operator- (const Vector<T> &vec1);
	Vector<T> operator- (const T& scalar);
	
	T operator* (const Vector<T>&) const;
	T& operator[] (const int) const;
	
	Vector<T>& operator= (const Vector<T>&);
	Vector<T>& operator= (const T&);
	
	
private:
	int DeleteMyData();
	T* elements;
	int length;
	int maxlength;
	int Delete();
	
};



template <class T>
Vector<T>::Vector(const int l)
{
  length = l;
  maxlength = l;
  elements = new T[maxlength+1];
}



template <class T>
Vector<T>::Vector(const Vector<T> & v1)
{
  length = v1.GetLength();
  maxlength = length;
  elements = new T[maxlength+1];
  
  for(int i=1; i<=length; i++)
    {
      elements[i] = v1.GetElement(i);
    }
}


template <class T>
template <class S> 
Vector<T>:: 
Vector(const  Vector<S>& v1)
{
  length = v1.GetLength();
  maxlength = length;
  elements = new T[maxlength+1];
  
  for(int i=1; i<=length; i++)
    {
      elements[i] = (T) v1.GetElement(i);
    }
}





template <class T>
Vector<T>::
~Vector()
{
	DeleteMyData();
}

template <class T>
int Vector<T>::Reset(const Vector<T> & vec)
{
  if(this == &vec)
    {
      return(0);
    }

  length = vec.GetLength();
  maxlength = length;
  delete [] elements;
  elements = new T[maxlength+1];

  for(int i=1; i<=length; i++)
    {
      elements[i] = vec.GetElement(i);
    }
}



template <class T>
int Vector<T>::Reset(const int newlength)
{

  if(newlength<0)
    {
      std::cout << "Vector::Reset: Cannot reset to negative length!" << std::endl;
      exit(1);
    }

  delete [] elements;

  length = newlength;
  maxlength = length;
  elements = new T[newlength + 1];

  return(0);
}



template <class T>
int Vector<T>::SetZero()
{
  for(int i=1; i<=length; i++)
    {
      elements[i] = 0;
    }

  return(0);
}



template <class T>
int Vector<T>::SetOne()
{
  for(int i=1; i<=length; i++)
    {
      elements[i] = 1;
    }

  return(0);
}



template <class T>
int Vector<T>::ExtendTo(const int newlength)
{
  T* newelements;

  if(newlength<maxlength)
    {
      std::cout << "Array::ExtendTo: Cannot extend to smaller length! Use ShrinkTo(...)!" << std::endl;
      exit(1);
    }

  newelements = new T[newlength + 1];

  for(int i=1;i<=length;i++)
    {
      newelements[i] = elements[i];
    }

  maxlength = newlength;
  delete [] elements;
  elements = newelements;

  return(0);
}



template <class T>
int Vector<T>::SetElement(const int position, const T& element)
{
#ifdef DEBUG
  if(position<=0)
    {
      std::cout << "Array::SetElement:Give positive position!" << std::endl;
      exit(1);
    };
#endif
    
  if(position>maxlength)
    {
      maxlength = int(position*1.41421);
      ExtendTo(maxlength);
    }
  
  if(position>length)
    {
      position = length;
    }

  //  cout << "element: " << element << ", ";

  elements[position] = element;

  //  cout << elements[position] << endl;

  return(0);
}


// added by Roland Schulte
template <class T>
int Vector<T>::Set(const T& entry)
{
  for(int i=1; i<=length; i++)
    {
      elements[i] = entry;   
    }

  return(0);
}



template <class T>
int Vector<T>::Add(const int i, const T& addend)
{
  return(SetElement(i,elements[i]+addend));
}


// added by Roland Schulte
template <class T>
int Vector<T>::Add(const T& addend)
{
  for(int i=1; i<=length; i++)
    {
      elements[i] += addend;   
    }

  return(0);
}


template <class T>
int Vector<T>::Multiply(const int i, const T& factor)
{
  return(SetElement(i,factor*elements[i]));
}


// added by Roland Schulte
template <class T>
int Vector<T>::Multiply(const T& factor)
{
  for(int i=1; i<=length; i++)
    {
     elements[i] = elements[i] * factor;   
    }

  return(0);
}


template <class T>
int Vector<T>::GetElement(const int position, T& element) const
{
#ifdef DEBUG
  if(position<=0 || position>length)
    {
      std::cout << "GetElement: Array position outside valid range!" << std::endl;
      exit(1);
    };
#endif

    element = elements[position];

  return(0);
}



template <class T>
T& Vector<T>::GetElement(const int position) const
{
  return(elements[position]);
}



template <class T>
T& Vector<T>::Get(const int position) const
{
#ifdef DEBUG
  if(position<=0 || position>length)
    {
      std::cout << "GetElement: Array position outside valid range!" << std::endl;
      exit(1);
    };
#endif
      return(elements[position]);
    
}



template <class T>
T& Vector<T>::operator[] (const int i) const
{
#ifdef DEBUG
  if(i<=0 || i>length)
    {
      std::cout << "Operator []: Array position outside valid range!" << std::endl;
      std::cout << "Tried to access position " << i << " where only 1 ... " << length << " are possible!" << std::endl; 
      exit(1);
    };
#endif
      return(elements[i]);
    
}







template <class T>
Vector<T>& Vector<T>::operator= (const Vector<T> & a)
{
  if(this == &a)
    {
      return(*this);
    }

  int lengthofa = a.GetLength();

  if(length != lengthofa)
    {
      this->Reset(lengthofa);
    }

  for(int i=1;i<=length;i++)
    {
      elements[i] = a[i];
    }

  return(*this);
}



template <class T>
Vector<T>& Vector<T>::operator= (const T& a)
{
  for(int i=1;i<=length;i++)
    {
      elements[i] = a;
    }

  return(*this);
}



template <class T>
Vector<T> Vector<T>::operator- (const T& scalar)
{
 Vector<T>	tempVec(length);

  for(int i=1;i<=length;i++)
    {
      tempVec[i] = elements[i]-scalar;
    }
	
  return tempVec;
}

template <class T>
Vector<T> Vector<T>::operator- (const Vector<T>& vec1)
{
 Vector<T>	tempVec(length);

  for(int i=1;i<=length;i++)
    {
      tempVec[i] = elements[i]-vec1[i];
    }
  return tempVec;
}

template <class T>
Vector<T> Vector<T>::operator+ (const Vector<T>& vec1)
{
 Vector<T>	tempVec(length);

  for(int i=1;i<=length;i++)
    {
      tempVec[i] = elements[i]+vec1[i];
    }
  return tempVec;
}






template <class T>
std::ostream & operator<< (std::ostream &o, const Vector<T> & a)
{
 
  o << a.GetLength() << std::endl;

  for(int i=1; i<=a.GetLength(); i++) 
    {
      o << a.GetElement(i) << " ";
    }

   o <<  std::endl;
   return o;
}


template <class T>
std::istream & operator>> (std::istream & i, Vector<T> & a)
{
  int l;
  T element;

  i >> l;
  a.Reset(l);

  for(int j=1; j<=l; j++) 
    {
      i >> element;
      a.SetElement(j,element);
    }

  return i;
}



template <class T>
int Vector<T>::DeleteMyData()
{
  delete [] elements;
  elements = NULL;
  length = 0;
  maxlength = 0;
  return 0;
}


template <class T>
Vector<T> operator* (float value,const Vector<T> vec1)
{
	int length = vec1.GetLength();
	Vector<T> tempVec(length);
	for(int i=1; i<=length; i++)
    {
      tempVec[i] = (T) vec1[i] * value;
    }
	return tempVec;
}


template <class T>
T NormSquared(const Vector<T> & vec1)
{
	int length = vec1.GetLength();
	float tempVal=0.0;
	for(int i=1; i<=length; i++)
    {
      tempVal +=  vec1[i] * vec1[i];
    }
	return tempVal;
}


template <class T>
float Norm(const Vector<T> vec1)
{
	return sqrt((float) NormSquared(vec1));
}




template <class T, class S> 
Vector<T> ComponentMultiply(const Vector<T>& A1, const Vector<S>& A2)
{
  Vector<T> B(3);
  int length;
  length=A1.GetLength();
  if (A2.GetLength()<length) length=A2.GetLength();

  for(int i=1;i<=length;i++)
    {
      B[i] =(double) A1[i]*A2[i];
    }

  return(B);

}




}

#endif


