//Itk Includes
#include "itktypes.h"




using namespace std;
#include "vector.h"



#include "compartment.h"

#include "compartmentlist.h"

#include "multicompartmentlist.h"




static int
EmptyCallBackFunction(int percentage)
{
    return 0;
}

int FillCellBody(InputImageType::Pointer inputImage, Vector<int> cellBodyPosition)
{
    unsigned int rangeX=20;
   unsigned int rangeY=20;
   unsigned int rangeZ=20;
    
  unsigned  int startX=cellBodyPosition[1]-rangeX;
   unsigned int startY=cellBodyPosition[2]-rangeY;
    unsigned int startZ=cellBodyPosition[3]-rangeZ;
   
  unsigned int stopX=cellBodyPosition[1]+rangeX;
   unsigned int stopY=cellBodyPosition[2]+rangeY;
   unsigned int stopZ=cellBodyPosition[3]+rangeZ;
    
    
    
    for ( unsigned int z=startZ;z<=stopZ;z++)
	for ( unsigned int y=startY;y<=stopY;y++)
	    for ( unsigned int x=startX;x<=stopX;x++)
	    {
		InputImageType::RegionType region;
		InputImageType::SizeType size;
	
	region= inputImage->GetRequestedRegion();
	size = region.GetSize();
	if ((x>=size[0] || y>=size[1] || z>=size[2]) || (x<0 || y<0 ||z<0))
			continue;

		InputImageType::IndexType pixelIndex;
		pixelIndex[0]=x;
		pixelIndex[1]=y;
		pixelIndex[2]=z;
		
		InputImageType::PixelType pixel;
		pixel=255;
		

		
		
		
		inputImage->SetPixel(pixelIndex,pixel); 
	    }
		

		return 0;

}



int DataImageInformation(InputImageType::Pointer inputImage)
{
	InputImageType::RegionType region;
  InputImageType::SizeType size;

  region= inputImage->GetRequestedRegion();
  size = region.GetSize();

  cout << "Data Imgae Size:" << size[0] << "," << size[1]  << "," << size[2] << endl;

  

  /* Check Max Pixel Values */
  unsigned int max=0;
  unsigned int min=32768;
  
  for (unsigned int z=0;z<size[2];z++)
    for (unsigned int y=0;y<size[1];y++)
      for (unsigned int x=0;x<size[0];x++)
	{

	  InputImageType::IndexType pixelIndex;
	  pixelIndex[0]=x;
	  pixelIndex[1]=y;
	  pixelIndex[2]=z;
  
	  InputImageType::PixelType pixelValue= inputImage->GetPixel(pixelIndex); 
	  if (max<pixelValue) max=pixelValue;
	if (min>pixelValue) min=pixelValue;
	}
  
  std::cout << "Max PixelValue=" << max  << std::endl;


	return 0;
}



int  CreateReconstruction(InputImageType::Pointer inputImage, Vector<int> cellBodyPosition, Cellbody **cellBody)
{
		int errorcode=0;
	//	int maxlevel;
	 /* Reconstruction first Part */
        MultiCompartmentList* multiCompartmentList=new MultiCompartmentList(inputImage,EmptyCallBackFunction);
        
        
         if (verbose) cout << "reconstruction::TreeGenerationDetectTerminations::One level approach, one voxel = one compartment" << endl;
        multiCompartmentList->createFinestCompartmentListFromScratch();
        if (verbose>2) cout << (* multiCompartmentList) << endl;
        cout << "Set Cellbody" << endl;
        errorcode=multiCompartmentList->SetBodyDirectFromFinestList(cellBodyPosition);
        if (errorcode) return errorcode;
	
	
        //*evaluate neigbour hood
        if (verbose) cout << "reconstruction::TreeGenerationDetectTerminations::Mark boundary compartments" << endl << flush;
        multiCompartmentList->MarkBoundaryCompartments();
	
		if (verbose) cout << "reconstruction::TreeGenerationDetectTerminations::SetNeigbours27()" << endl;
        multiCompartmentList->SetNeigbour27(); 
	
        if (verbose) cout << "reconstruction::TreeGenerationDetectTerminations::Calculate and set distances from cellbody using champfer function" << endl << flush;
        multiCompartmentList->CalcualteAndSetDistancesFromBodyCompartmentUsingChamferFunction(); 
        
        if (verbose) cout << "reconstruction::TreeGenerationDetectTerminations::Search for dendrite terminations" << endl << flush;
        multiCompartmentList->SearchLokalMaximaOfDistance27AndCreateButtonCompList();
        
	
        cout << "reconstruction::TreeGenerationDetectTerminations::Check if dendrite terminations are meaningfull" << endl << flush;
		multiCompartmentList->CheckIfButtonsInButtonCompListAreMeaningfullAndDeleteOther(20,5);
        //usually dendrite terminations are 10qm from soma
        //usually they are longer than 3qm
           
    
		if (verbose) cout << "Reconstruction::TreeGenerationCreateNervecell::Skeletonization" <<  endl << flush;
		multiCompartmentList->ExtractSkeleton2(); 
	
        if (verbose) cout << "Reconstruction::TreeGenerationCreateNervecell::Detect crosssings and write neuron data set." << endl;
        if (multiCompartmentList->WalkFromButtonsToBodyAndCreateNeuronStructure())
        {
            cout << "Error durring reconstruction" << endl;
            return 1;
        }
        	
		if (verbose) cout << "Reconstruction::TreeGenerationCreateNervecell::MeasureRadii" << endl;
        (multiCompartmentList->GetCellBody())->measureRadii(100); 
		if (verbose) cout << "Finished measure radius" << endl << flush << endl;
	
		if (verbose) cout << "Reconstruction::TreeGenerationCreateNervecell::Finished Reconstruction!" << endl;
	
		(*cellBody)=multiCompartmentList->GetCellBody();
		
		return  0;   
 

}




/* Should be a class one time */

	char inputFileNameBuffer[1024];
	double dx, dy, dz;
	int px, py, pz;
	char hocFileNameBuffer[1024];



int
WriteHocFile(Cellbody *cellBody)
{
  
  
  cout << "WriteHocFile:" << hocFileNameBuffer  << endl;
    ofstream neuronData(hocFileNameBuffer);
    if (!neuronData) return CANTWRITEHOCFILE;
    neuronData << "/* This hoc file was created using Dendrite Tracker */" << endl;
    neuronData << "/* Dendrite Tracker is programmed by Philip J. Broser, Max Planck Institut for medical research Heidelber, Germany */" << endl;
    neuronData << "/* For information about Dendrite Tracker see http://www.mpimf-heidelberg.mpg.de/~pbroser/  .*/" << endl;
    neuronData << "/* This file is based on data file:" <<   inputFileNameBuffer << " */" << endl;
    neuronData << "/* Reconstruction Settings: */" << endl;
    neuronData << "minLengthOfDendriteInQm = " << 5 << endl;
    neuronData << "minDistanceFromBodyPositionInQm = " << 20 << endl;
    neuronData << "px=" << px << endl;
    neuronData << "py=" << py << endl;
    neuronData << "pz=" << pz << endl;
    neuronData << "dx=" << dx << endl;
    neuronData << "dy=" << dy << endl;
    neuronData << "dz=" << dz << endl;
    neuronData << "/* Morphology */" << endl;
    neuronData <<  *(cellBody);

	
    return 0;
};




int main( int argc , char * argv[])
{ 
	cout << "This is the DendriteTracker " << endl;
	cout << "Programmed by Philip J. Broser at the Max Planck Institut for medical Research" << endl;
	cout << "Contact: philip.broser@mpimf-heidelberg.mpg.de" << endl;
	cout << "You will need to enter" << endl;
	cout << "1. Input Image File Name (which should be a multilayer tiff, with foreground 255 and background 0)" << endl;
	cout << "2. VoxelSize in qm in x,y,z direction" << endl;
	cout << "3. Cellbodyposition in Image Coordinates. You can use ImageJ to estimate the position." << endl;
	cout << "4. Name of the OutPut HocFile." << endl;
	cout << endl;
	


	/*TestData*/
	sprintf(inputFileNameBuffer,"TestData.tif");
	dx=0.36;dy=0.36;dz=0.36;
	px=216;py=254;pz=19;
	px=226;py=280;pz=15;
	
	sprintf(hocFileNameBuffer,"TestData.hoc");
	//int correct=0;
	


	
	int correct=1;
	while (correct)
	{
	//Input File Name:
	cout << "Input Image File Name:";
	cin >> inputFileNameBuffer;
	cout << endl;

	cout << "VoxelSize Fixed to 0.36qm in x,y,z direction" << endl;
	/*
	cout << "Voxel Size in qm in" << endl << "x direction:"; 
	cin >> dx;
	cout << "y direction:" ; cin >> dy;
	cout << "z direction:" ; cin >> dz;
	cout << endl;
	cout << endl;
	*/
	dx=0.36;
	dy=0.36;
	dz=0.36;
	//Position
	
	cout << "Cellbody Position Image Coordinates" << endl << "px:"; cin >> px;
	cout << "py:" ; cin >> py;
	cout << "pz:" ; cin >> pz;
	cout << endl;
	
	cout << endl;
	
	//Hoc File Name
	cout << "Hoc File Name:";
	cin >> hocFileNameBuffer;
	
	
	cout << endl << endl << "Input File Name is:" << inputFileNameBuffer << endl;
	cout << "Spacing:" << dx << "," << dy << "," << dz << "*  10 e-6 m" << endl;
	cout << "Cellbody Position:" << px << "," << py << "," << pz << endl;
	cout << endl << "Hoc File Name is:" << hocFileNameBuffer << endl << endl;
	
	cout << "(A) ccept , (C)hange , (E) xit ?";
	char answer;
	cin >> answer;
	if (answer=='A' || answer=='a') correct=0;
	
	if (answer=='E' || answer=='e') 
		{
			cout << "Good Bye!" << endl;
			return 0;
		}
	}
	






//Image Loader and Writer
  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();


//Data Specifi Parameter
Vector<int> cellBodyPosition(3);

// Read Data Image
  
  reader->SetFileName( inputFileNameBuffer);
std::cout << "Load Image File" << std::endl << std::flush; 	
try
{
  reader->Update();
}
catch ( itk::ExceptionObject &err )
    { 
      std::cout << "ReaderExeptionObject caught !" << std::endl;
      std::cout << err << std::endl;
      return -1;
    }


  InputImageType::Pointer inputImage=reader->GetOutput();
	std::cout << "Load Image File: done" << std::endl << std::flush; 
InputImageType::SpacingType spacing;
spacing[0] = dx; // spacing along X
spacing[1] = dy; // spacing along Y
spacing[2] = dz; // spacing along Z
inputImage->SetSpacing( spacing );

 DataImageInformation(inputImage);
 Cellbody *cellBody;
 cellBodyPosition[1]=px;
 cellBodyPosition[2]=py;
 cellBodyPosition[3]=pz;
 
 FillCellBody(inputImage, cellBodyPosition);
 int errorcode=CreateReconstruction(inputImage, cellBodyPosition, &cellBody);
 if (errorcode) return errorcode;
 
  errorcode=WriteHocFile(cellBody);
  if (errorcode) return errorcode;
 
 
  cout << "Finished Dendrite Tracker sugsesfull!" << endl;
  cout << "Have a nice day!" << endl;
  cout << "Greetings Philip" << endl;
  
  return 0;
}



